@@ -15,8 +15,17 @@ use key::DidKeyResolver;
 use base58::FromBase58;
 pub use did_key::{Document, KeyFormat, VerificationMethod};
 use error::Error;
+use lazy_static::lazy_static;
+use regex::Regex;
 use serde::{Deserialize, Serialize};
 
+lazy_static! {
+    static ref DID_REGEX: Regex = Regex::new(
+        r"(?x)(?P<prefix>[did]{3}):(?P<method>[a-z]*):(?P<key_id>[-_a-zA-Z0-9]*)([:?/]?)(S)*??",
+    )
+    .unwrap();
+}
+
 /// # Universal trait for DID document resolver.
 /// Standardises signature for resolver output.
 ///
@@ -115,7 +124,7 @@ impl DdoParser for Document {
 /// Output is `Document` or `Error`.
 ///
 pub fn try_resolve_any(did_url: &str) -> Result<Document, Error> {
-    let re = regex::Regex::new(r"^((?P<prefix>did){1}:(?P<method>[a-z]*){1}:(?P<id>.+?))((?P<kerlid>\?kerl=)(?P<kerl>[a-zA-Z0-9]+?))?$").unwrap();
+    let re = regex::Regex::new(r"^((?P<prefix>did){1}:(?P<method>[-_A-Za-z0-9]*){1}:(?P<id>.+?))((?P<kerlid>\?kerl=)(?P<kerl>[a-zA-Z0-9]+?))?$").unwrap();
     match re.captures(did_url) {
         Some(caps) => {
             match &caps["method"] {
@@ -149,7 +158,7 @@ pub fn try_resolve_any(did_url: &str) -> Result<Document, Error> {
 /// Output is Option: `Some(Document)` or `None`. Will never fail with error.
 ///
 pub fn resolve_any(did_url: &str) -> Option<Document> {
-    let re = regex::Regex::new(r"^((?P<prefix>did){1}:(?P<method>[a-z]*){1}:(?P<id>.+?))((?P<kerlid>\?kerl=)(?P<kerl>[a-zA-Z0-9]+?))?$").unwrap();
+    let re = regex::Regex::new(r"^((?P<prefix>did){1}:(?P<method>[-_a-zA-Z0-9]*){1}:(?P<id>.+?))((?P<kerlid>\?kerl=)(?P<kerl>[a-zA-Z0-9]+?))?$").unwrap();
     match re.captures(did_url) {
         Some(caps) => {
             let resolver: Box<dyn DdoResolver> = match &caps["method"] {
@@ -207,11 +216,7 @@ pub(crate) fn get_public_key(doc: &Document, curve: &str) -> Option<KeyFormat> {
 // Helper function to get key id from did url
 // # + id
 pub(crate) fn key_id_from_didurl(url: &str) -> String {
-    let re = regex::Regex::new(
-        r"(?x)(?P<prefix>[did]{3}):(?P<method>[a-z]*):(?P<key_id>[a-zA-Z0-9]*)([:?/]?)(S)*??",
-    )
-    .unwrap();
-    match re.captures(url) {
+    match DID_REGEX.captures(url) {
         Some(s) => match s.name("key_id") {
             Some(name) => format!("#{}", name.as_str()),
             None => String::default(),
@@ -220,6 +225,18 @@ pub(crate) fn key_id_from_didurl(url: &str) -> String {
     }
 }
 
+// Parses and String formats prefix:method:key_id from given &str
+//
+pub fn did_id_from_url(url: &str) -> Option<String> {
+    let captures = DID_REGEX.captures(url)?;
+    Some(format!(
+        "{}:{}:{}",
+        captures.name("prefix")?.as_str(),
+        captures.name("method")?.as_str(),
+        captures.name("key_id")?.as_str()
+    ))
+}
+
 /// "Temporary" struct to extend did_key crate's `Document` with `KeyAgreement` instead of string.
 ///
 #[cfg(feature = "didkey")]
@@ -231,3 +248,29 @@ pub struct KeyAgreement {
     #[serde(rename = "publicKeyBase58")]
     pub public_key_base58: String,
 }
+
+#[test]
+fn did_id_from_url_test() {
+    let keri = "did:keri:someiderNTIFIER2345432?bunch_of_niose!_$(#)";
+    let keri_sym = "did:keri:D1bkcOzM-YwEXKPc5yHbMzkHRrZS3O6QAVEpGsS0XpF_E";
+    let key = "did:key:bu03rlnth4gpk09y4cr3DEGCTHUDGc45RCGUCH?again_some_rubbish";
+    let long = "did:verylongid:BXDHCG8765678THDIYFGCRNWMBXIF34543HDGC?MOREnoise_?";
+    let not_a_did = "thisisnot_a_did";
+    assert_eq!(
+        &did_id_from_url(keri).unwrap(),
+        "did:keri:someiderNTIFIER2345432"
+    );
+    assert_eq!(
+        &did_id_from_url(keri_sym).unwrap(),
+        "did:keri:D1bkcOzM-YwEXKPc5yHbMzkHRrZS3O6QAVEpGsS0XpF_E"
+    );
+    assert_eq!(
+        &did_id_from_url(key).unwrap(),
+        "did:key:bu03rlnth4gpk09y4cr3DEGCTHUDGc45RCGUCH"
+    );
+    assert_eq!(
+        &did_id_from_url(long).unwrap(),
+        "did:verylongid:BXDHCG8765678THDIYFGCRNWMBXIF34543HDGC"
+    );
+    assert!(did_id_from_url(not_a_did).is_none());
+}
